//
//  BibItem.h
//  BibCompanion
//
//  Created by Hitoshi Nakahara on 04/11/15.
//  Copyright 2004 Hit-C Soft House. All rights reserved.
//

#import <Cocoa/Cocoa.h>

@class	RealDatabase, MacroList;

#ifndef BCListFormatType

typedef enum BCListFormatType {
    BCListFormatType_bibKey		= 0,
    BCListFormatType_cite,
	BCListFormatType_nocite,
	BCListFormatType_bibitem,
	BCListFormatType_html,
	BCListFormatType_tab,
	BCListFormatType_rtf,
	BCListFormatType_plain
} BCListFormatType;

typedef enum BCFileState {
	BCFileState_NoFile,
	BCFileState_NotFound,
	BCFileState_ExistsLocal,
	BCFileState_ExistsAsFolder,
	BCFileState_ExistsRemote
} BCFileState;

typedef enum BC_AuthorNameOrder {
	BC_AuthorNameOrder_FamilyFirst = 0,
	BC_AuthorNameOrder_GivenFirst,
	BC_AuthorNameOrder_FamilyOnly
} BC_AuthorNameOrder;


#define		BC_BibItemPboardType		@"BibCompanionBibItemPboardType"

#define		BC_ManualLibProperty_Description	@"Description"
#define		BC_ManualLibProperty_BibItems		@"Bib Item List"
#define		BC_ManualLibProperty_PromisedID		@"PromisedID"

#endif

@interface BibItem : NSObject <NSCoding> {

	NSString			*bibType;		// immutable, all upper case
	NSMutableString		*bibKey;
	NSString			*originalBibKey;
	NSString			*originalMatchKey;

	NSMutableDictionary	*bibFieldsElements;		// bib field dictionary which contains NSMutableArray of BibElement
	NSMutableDictionary	*bibFieldsForEdit;		// bib field dictionary ready for edit
	NSMutableDictionary	*bibFieldsForDisplay;	// same as the above except for TeX->unicode conversion
	NSMutableDictionary	*bibFieldsForListView;	// NSAttribStr's for list view
	
	NSArray				*authorList;			// array of AuthorName
	NSArray				*editorList;			// array of AuthorName
	BOOL				noAuthor;
	NSArray				*originalAuthorList;
	NSArray				*originalEditorList;
	
	RealDatabase		*masterDatabase;
	NSString			*promisedDbID;

	NSString			*sortIdentifier;
	BOOL				sortAscending;
	
	BCFileState			fileState;
	NSImage				*fileIcon;
	
	BOOL				dirty;
}

- (id)init;
- (id)initBibItemOfType:(NSString*)type bibKey:(NSString*)key
		andData:(NSMutableDictionary*)fields forDatabase:(RealDatabase*)database forceAbsolutePath:(BOOL)forceAbsolutePath;
- (id)initBibItemOfKey:(NSString*)key withProperty:(NSDictionary*)fieldProperty;
- (void)dealloc;

+ (id)bibItemOfType:(NSString*)type bibKey:(NSString*)key
		andData:(NSMutableDictionary*)fields forDatabase:(RealDatabase*)database;

+ (id)newBibItemOfType:(NSString*)type bibKey:(NSString*)key
		forDatabase:(RealDatabase*)database;
+ (id)bibItemOfKey:(NSString*)key withProperty:(NSDictionary*)fieldProperty;

//+ (id)bibItemWithBibItem:(BibItem*)sourceItem withBibKey:(NSString*)key forDatabase:(RealDatabase*)database;

- (void)copyDataFromItem:(BibItem*)sourceItem;

- (BOOL)isEditable;
- (BOOL)canReadFile;
- (BOOL)canWriteFile;

- (void)expandWithMacro:(MacroList*)macroList;
- (NSString*)bibType;
- (NSString*)bibKey;
- (NSString*)sortKey;
- (NSString*)originalBibKey;
- (NSMutableDictionary*)bibFieldsElements;
- (NSDictionary*)bibFieldsForEdit;
- (NSDictionary*)bibFieldsForDisplay;
- (NSDictionary*)bibFieldsForListView;
- (NSString*)rawField;
- (NSString*)rawFieldWithLF:(NSString*)lineFeed andTab:(BOOL)preTab;
- (NSAttributedString*)valueForColumn:(NSString*)identifier;

- (NSString*)fieldDataForView:(NSString*)field;
- (NSString*)fieldDataForEdit:(NSString*)field;
- (NSString*)fieldDataForListFormat:(NSString*)field;
- (NSString*)nameListForListFormat:(NSString*)nameListString;
- (NSString*)abbrevString:(NSString*)inString;

- (BOOL)strayItem;
- (RealDatabase*)database;
- (void)setDatabase:(RealDatabase*)database;
//- (void)promisedDbID:(NSString*)name;
//- (void)databaseNameChanged;
- (void)databaseClosed:(RealDatabase*)database;
- (BibItem*)identicalItemInDatabase:(RealDatabase*)database;

- (NSArray*)authorList;
- (NSArray*)editorList;
- (NSArray*)originalAuthorList;
- (NSArray*)originalEditorList;

- (void)makeNameLists;
- (NSString*)makeAuthorsForList:(BC_AuthorNameOrder)format;
- (NSString*)makeEditorsForList:(BC_AuthorNameOrder)format;
- (NSString*)makePagesForList;
- (NSString*)makeModifiedForList;
- (void)makeRawField;
- (void)makeAttributedStringForList;
- (void)makeAttributedStringForIdentifier:(NSString*)identifier;

- (void)setSortIdentifier:(NSString*)identifier ascending:(BOOL)ascend;
- (NSComparisonResult)sortByBibkey:(BibItem*)anotherName;
- (NSComparisonResult)sortByIdentifier:(BibItem*)anotherName;

- (BOOL)containsString:(NSString*)searchStr inFields:(NSArray*)fieldList options:(unsigned)mask;

- (BCFileState)fileState;
- (BCFileState)checkFileState;
- (NSImage*)fileIcon;
- (NSString*)attachedFile;
- (NSURL*)attachedURL;
- (NSString*)attachedEmail;

- (void)clearDirty;
- (void)touch;
- (void)fixBibKeyOnly;
- (void)fixChanged;
- (BOOL)isDirty;

- (BOOL)setFieldData:(NSString*)data forField:field;
- (BOOL)checkRawValidity:(NSString*)inStr itemElements:(NSMutableDictionary**)fields;
- (BOOL)setRawFields:(NSString*)data;
- (void)changeBibKey:(NSString*)newKey;

- (void)releaseBackup;
- (void)backupOriginal;
- (void)backupOriginalBibKey;
- (BOOL)namesWereChanged;

- (NSString*)bibKeyFromGenRule:(NSString*)genRule;
- (NSString*)stringFromGenRule:(NSString*)genRule withFormat:(BCListFormatType)format;
- (NSString*)uniqueBibKey:(NSString*)baseKey;
- (NSString*)generateBibKey;
- (NSString*)makeItemsTextWithFormat:(BCListFormatType)format withIndex:(int)index;
//- (NSAttributedString*)makeItemsAttributedTextWithIndex:(int)index attributes:(NSDictionary*)normalAttrib;

- (BOOL)mark;
- (void)setMark:(BOOL)mark;

- (NSDictionary*)bibItemDictionaryForProperty;
- (NSString*)bibItemStringWithLF:(NSString*)lineFeed;

- (void)remakeFilePath:(BOOL)full;
- (void)remakeDescription:(NSString*)newDescription;

- (void)setAttachment:(NSString*)newPath;
- (NSString*)appendAttachment:(NSString*)file attractive:(BOOL)attractive;
- (NSString*)appendAttachments:(NSArray*)files attractive:(BOOL)attractive;
- (void)setModified;

- (NSString*)matchKey:(BOOL)useOriginal;

@end
