/* DBListController */

#import <Cocoa/Cocoa.h>
#import	"BibDatabase.h"

@class	BCMainController, BibDatabase, DatabaseGroup, RealDatabase, LocalDatabase, ManualLibrary;
@class	BibItem, DbListOutlineView;

@interface DBListController : NSObject
{
    IBOutlet BCMainController	*mainDocument;
    IBOutlet DbListOutlineView	*tableDatabase;
    IBOutlet NSPopUpButton		*popupAppendDB;
    IBOutlet NSProgressIndicator *indicatorLoadingFile;
    IBOutlet NSProgressIndicator *indicatorWaitAWhile;
    IBOutlet NSTextField		*statusDatabase;
    IBOutlet NSButton			*buttonAppendItem;
    IBOutlet NSMenu				*menuFileMenu;
    IBOutlet NSMenu				*menuDbContextMenu;

    IBOutlet NSWindow			*mainWindow;

	BibDatabase		*rootNode;
	DatabaseGroup	*databaseNode;
	
	NSMutableArray	*loadingDatabase;
	NSMutableArray	*finishedDatabase;
	NSMutableArray	*waitingDatabase;
	NSMutableArray	*waitingLibraries;
	NSTimer			*loadingCheckTimer;
	BOOL			loadingListLocked;
	
	BibDatabase		*initialSelection;
	BOOL			initialLoading;
	
	float			maxColumnWidth;
	
	NSTimer			*autoSaveTimer;

	BibDatabase		*draggingSourceItem;
	
	NSLock			*loadingListLock;
}

+ (DBListController*)globalDBListController;

- (IBAction)paste:(id)sender;
- (IBAction)openLocalDatabase:(id)sender;
- (IBAction)openRemoteDatabase:(id)sender;
- (IBAction)createNewDatabase:(id)sender;
- (IBAction)createLibraryGroup:(id)sender;
- (IBAction)createSmartLibrary:(id)sender;
- (IBAction)createManualLibrary:(id)sender;
- (IBAction)removeFromList:(id)sender;
- (IBAction)openPropertySheet:(id)sender;
- (IBAction)appendBibItem:(id)sender;
- (IBAction)stopLoadingDB:(id)sender;
- (IBAction)saveDocument:(id)sender;
- (IBAction)saveSelection:(id)sender;
- (IBAction)saveSelectionAs:(id)sender;
- (IBAction)editMacros:(id)sender;
- (IBAction)editPreambles:(id)sender;
- (IBAction)updateSmartLib:(id)sender;
- (IBAction)updateAllSmartLibs:(id)sender;
- (IBAction)searchDuplicates:(id)sender;

- (void)notified:(NSNotification*)notification;
- (void)selectionChanged;
//- (void)createAttributedNames;
- (void)reload:(BOOL)selChange;
- (void)reloadWithDatabase:(BibDatabase*)db;

- (BibDatabase*)newItemParent;

- (void)openLocalDidEnd:(NSOpenPanel*)panel returnCode:(int)returnCode contextInfo:(void*)contextInfo;

- (float)maxColumnWidth;

- (BOOL)selectDatabase:(BibDatabase*)selDb;
- (BOOL)selectDatabaseID:(NSString*)dbID ofKind:(BCDatabaseKind)dbKind;
- (BibDatabase*)selectedDatabase;
- (RealDatabase*)findDatabaseOfID:(NSString*)id;

- (void)getNumLocal:(int*)numLocal andNumRemote:(int*)numRemote andNumSmart:(int*)numSmart andNumManual:(int*)numManual;

//- (void)sortDatabases;
- (void)removeDatabase:(BibDatabase*)removeDB;

- (LocalDatabase*)appendLocalDatabase:(NSString*)dbPath canCreate:(BOOL)canCreate;

- (void)refleshStatus;

- (BOOL)isLoading;
- (void)setLibraryForPostProcess:(ManualLibrary*)library;
- (void)databaseStartLoading:(BibDatabase*)loadingDB;
- (BOOL)databaseFinishedLoading:(BibDatabase*)loadingDB;
- (void)loadingPostProcess:(BibDatabase*)loadedDB;
//- (void)keepLoadingLists;
//- (void)releaseLoadingLists;

- (NSArray*)addCopyOfBibItems:(NSArray*)bibItems toDatabase:(BibDatabase*)targetDB;
- (int)removeBibItems:(NSArray*)bibItems removeFile:(BOOL)removeFile;
- (void)removeItemFromLibraries:(BibItem*)bibItem;

- (void)replaceAuthorsForLib:(BibItem*)bibItem;
- (void)appendNewItemForSmartLib:(BibItem*)bibItem;

- (void)newBibItemOfType:(NSString*)bibtype forDatabase:(RealDatabase*)database;

- (NSArray*)getRealDbListExceptFor:(id)exceptItem;

- (void)bibItemChanged:(BibItem*)item;

- (void)prefsChanged;

- (void)reloadDatabases;
- (void)saveCurrentSettings;

- (BOOL)saveDocuments;
- (int)unsavedDatabases:(NSMutableString**)databaseNames;
- (void)startAutoSave;
- (void)stopAutoSave;

- (BOOL)initialLoading;

@end
