/* ItemListController */

#import <Cocoa/Cocoa.h>
#import	"BibItem.h"

@class	BCMainController, DBListController, DetailInfoController, FileViewController, BibItem;
@class	ItemListTableView, BibDatabase;

#ifndef BC_LastPboardAction
typedef enum BC_LastPboardAction {
	BC_LastPboardAction_cut = 0,
	BC_LastPboardAction_copy
} BC_LastPboardAction;
#endif

@interface ItemListController : NSObject
{
    IBOutlet BCMainController	*mainDocument;
	IBOutlet DBListController	*controllerDbList;

    IBOutlet ItemListTableView	*tableItemList;
    IBOutlet NSScrollView		*scrollBibItems;
    IBOutlet NSTextField		*statusBibItems;
    IBOutlet NSSearchField		*stringSearch;
    IBOutlet NSButton			*checkCaseSensitive;
    IBOutlet NSMenu				*searchMenu;
	
	IBOutlet DetailInfoController	*detailInfoController;
	IBOutlet FileViewController		*fileViewController;

    IBOutlet NSProgressIndicator *indicatorWaitAWhile;

	NSArray				*allItems;			// array of BibItem's
	NSMutableArray		*displayedItems;	// array of BibItem's
	
	NSString			*lastSortedColumn;
	BOOL				lastSortedAscend;
	
	NSMutableDictionary	*fieldNameList;			// dictionary of field names as target

	BC_LastPboardAction	lastPboardAction;
	NSArray				*itemListForLazyCopy;
}

- (IBAction)cut:(id)sender;
- (IBAction)copy:(id)sender;
- (IBAction)paste:(id)sender;
- (IBAction)undo:(id)sender;

- (IBAction)copyAsCite:(id)sender;
- (IBAction)copyAsNocite:(id)sender;
- (IBAction)copyAsBibkey:(id)sender;
- (IBAction)copyAsBibitemList:(id)sender;
- (IBAction)copyAsHTMLList:(id)sender;
- (IBAction)copyAsRTFList:(id)sender;
- (IBAction)copyAsTabList:(id)sender;
- (IBAction)copyAsTextList:(id)sender;
- (IBAction)createBibSubset:(id)sender;

- (IBAction)checkSelection:(id)sender;
- (IBAction)uncheckSelection:(id)sender;
- (IBAction)checkAll:(id)sender;
- (IBAction)uncheckAll:(id)sender;
- (IBAction)selectChecked:(id)sender;
- (IBAction)selectUnchecked:(id)sender;

- (IBAction)copyAttachments:(id)sender;
- (IBAction)openAttachments:(id)sender;
- (IBAction)openURLs:(id)sender;
- (IBAction)sendEmails:(id)sender;
- (IBAction)openFile:(id)sender;
- (IBAction)openURL:(id)sender;
- (IBAction)createNewMail:(id)sender;

- (IBAction)removeBibItem:(id)sender;
- (IBAction)searchTargetFieldChanged:(id)sender;
- (IBAction)searchTargetCheckAll:(id)sender;
- (IBAction)searchTargetUncheckAll:(id)sender;
- (IBAction)fieldSearch:(id)sender;
- (IBAction)searchCaseChanged:(id)sender;
- (IBAction)generateSelectedBibKeys:(id)sender;

- (IBAction)tableDoubleAction:(id)sender;

- (void)notified:(id)notification;
- (void)selectionChanged;
- (BOOL)validateMenuItem:(id)menuItem;

- (void)showDetail:(BOOL)show;

- (int)totalItems;
- (int)displayedItems;

- (void)setItems:(NSArray*)list;	// NSArray of BibItem's

- (void)refleshItemList;

- (void)refleshStatus;

- (BibItem*)firstSelection;
- (NSArray*)selectedItems;

- (void)mergeFieldNameList:(BibDatabase*)database;
//- (NSMutableDictionary*)fieldNameList;

- (int)selectBibItems:(NSArray*)items;
- (int)selectBibItem:(BibItem*)item;

- (NSArray*)selectedBibKeys;
- (void)selectItemsByBibkeys:(NSArray*)keys;

- (NSString*)createSelectionListIn:(NSArray*)items containsFiles:(BOOL*)haveFiles;
- (NSString*)createLockedListIn:(NSArray*)items num:(int*)numLocked ignoreStray:(BOOL)ignoreStray;
- (void)lockedAlertWith:(NSString*)lockedList numLocked:(int)numLocked;
- (int)confirmContinueWith:(NSString*)items numItems:(int)numItems
		title1:(NSString*)title1 message1:(NSString*)message1 title:(NSString*)title message:(NSString*)message
		alternateButton:(NSString*)alternateButton otherButton:(NSString*)otherButton;

- (NSString*)makeItemsText:(NSArray*)items withFormat:(BCListFormatType)format;
//- (NSAttributedString*)makeItemsAttributedText:(NSArray*)items;

- (NSArray*)makeFileListOf:(NSArray*)itemList;
- (NSArray*)makeUrlListOf:(NSArray*)itemList;
- (NSArray*)makeEmailListOf:(NSArray*)itemList;

- (void)copyItems:(NSArray*)itemList intoPaseteboard:(NSPasteboard*)pboard lazyCopy:(BOOL)lazy;
- (BC_LastPboardAction)lastPboardAction;

- (void)prefsChanged;
- (void)saveCurrentSettings;

- (void)clearIncrementalSearch;

- (void)becomeFirstResponder;

@end

