//
//  LocalDatabase.h
//  BibCompanion
//
//  Created by Hitoshi Nakahara on 04/11/18.
//  Copyright 2004 Hit-C Soft House. All rights reserved.
//

#import <Cocoa/Cocoa.h>
#import	"RealDatabase.h"

@class	StringInputStream, BibElementArray, AuthorName, BibItem;

#ifndef	BCLocalShareMode

/*typedef	enum	BC_LocalShareMode {
	BC_LocalShareMode_No = 0,
	BC_LocalShareMode_ReadOnly,
	BC_LocalShareMode_ReadWrite
} BC_LocalShareMode;

typedef	enum	BC_LocalShareFileMode {
	BC_LocalShareFileMode_No = 0,
	BC_LocalShareFileMode_ReadOnly,
	BC_LocalShareFileMode_ReadWrite
} BC_LocalShareFileMode;*/

typedef enum	BC_LocalLineEnding {
	BC_LocalLineEnding_CR = 0,
	BC_LocalLineEnding_CRLF,
	BC_LocalLineEnding_LF
} BC_LocalLineEnding;

typedef enum	BC_AttachFiles {
	BC_AttachFiles_Link = 0,
	BC_AttachFiles_Copy,
	BC_AttachFiles_Move,
	BC_AttachFiles_Alias
} BC_AttachFiles;

#define		BibDb_PrefKey_Encoding			@"textencoding"
#define		BibDb_PrefKey_LineMode			@"linemode"
#define		BibDb_PrefKey_Description		@"description"
#define		BibDb_PrefKey_DatabaseID		@"database_ID"
#define		BibDb_PrefKey_AttachmentMode	@"attachment_mode"
#define		BibDb_PrefKey_AttachmentPath	@"attachment_path"
#define		BibDb_PrefKey_RenameAfterBibkey	@"rename_after_bibkey"
#define		BibDb_PrefKey_KeepFullPath		@"keep_full_path"
#define		BibDb_PrefKey_PrivilegedHost	@"privileged_host"
#define		BibDb_PrefKey_PrivilegedUser	@"privileged_user"
#define		BibDb_PrefKey_LastSaved			@"last_saved"

#endif

@class	LocalDatabase;

@interface LocalDatabaseProperties : RealDatabaseProperties {

	NSString				*fullPath;			// full path
	NSString				*filePath;			// database file directory
	NSString				*fileName;			// file name

	NSStringEncoding		encoding;			// encoding
	BC_LocalLineEnding		lineEnding;			// line ending code
	
	BC_AttachFiles			fileAttachmentMode;	// file attachment mode
	NSString				*attachmentPath;	// attachment folder path
	BOOL					renameAfterBibkey;	// rename attachment after bibkey
	BOOL					keepFullPath;		// always keep full path
	
	NSString				*privilegedHost;	// name of a privileged host
	NSString				*privilegedUser;	// name of a privileged user
}

+ (LocalDatabaseProperties*)propertiesWithDatabase:(BibDatabase*)db;
+ (LocalDatabaseProperties*)propertiesWithDictionary:(NSDictionary*)propertyDic database:(BibDatabase*)lib;
+ (LocalDatabaseProperties*)propertiesWithProperties:(LocalDatabaseProperties*)source;

- (id)initWithDatabase:(BibDatabase*)db;
- (void)readFromDictionary:(NSDictionary*)propertyDic;
- (void)makeCopyOf:(LocalDatabaseProperties*)source;
- (NSMutableDictionary*)makePropertyDictionary;

- (NSString*)fullPath;
- (NSString*)filePath;
- (NSString*)fileName;
- (NSString*)description;
- (NSString*)relativeFilePath;
- (void)setFullPath:(NSString*)aFullPath;

- (NSString*)fullPathOfFile:(NSString*)file;
- (NSString*)relativePathOfFile:(NSString*)file;

- (NSStringEncoding)encoding;
- (void)setEncoding:(NSStringEncoding)anEncoding;
- (BC_LocalLineEnding)lineEnding;
- (void)setLineEnding:(BC_LocalLineEnding)aLineEnding;

- (BC_AttachFiles)fileAttachmentMode;
- (void)setFileAttachmentMode:(BC_AttachFiles)value;
- (NSString*)attachmentPath;
- (void)setAttachmentPath:(NSString*)value;
- (BOOL)renameAfterBibkey;
- (void)setRenameAfterBibkey:(BOOL)value;
- (BOOL)keepFullPath;
- (void)setKeepFullPath:(BOOL)value;

- (NSString*)privilegedHost;
- (void)setPrivilegedHost:(NSString*)value;
- (NSString*)privilegedUser;
- (void)setPrivilegedUser:(NSString*)value;
- (BOOL)hasPrivilegeToWrite;

@end

/////////////////

@interface LocalDatabase : RealDatabase {
	BOOL				loadingCompleted;
	NSMutableArray		*readingErrors;
	
	BOOL				prevKeepFullPath;
	NSString			*prevDescription;
}

+ (LocalDatabase*)localDatabaseWithFile:(NSString*)file canCreate:(BOOL)canCreate;
+ (LocalDatabase*)localDatabaseWithDictionary:(NSDictionary*)propertyDic;

- (id)initWithFile:(NSString*)file canCreate:(BOOL)canCreate;
- (id)initWithDictionary:(NSDictionary*)propertyDic;

- (BOOL)loadFromFile:(NSString*)file canCreate:(BOOL)canCreate;
- (void)startLoading;
- (void)loadFileInThread:(void*)userData;

- (NSString*)fullPath;
- (NSString*)filePath;
- (NSString*)fileName;

- (NSStringEncoding)encoding;
- (BC_AttachFiles)fileAttachmentMode;
- (void)setFileAttachmentMode:(BC_AttachFiles)value;
- (NSString*)attachmentPath;
- (void)setAttachmentPath:(NSString*)value;
- (BOOL)renameAfterBibkey;
- (void)setRenameAfterBibkey:(BOOL)value;
- (BOOL)keepFullPath;

- (void)openProperty;
- (void)propertiesChanged;

- (BOOL)readFromStream:(StringInputStream*)dataStream;
- (NSString*)readBlockHeaderFromStream:(StringInputStream*)dataStream;
- (void)skipCommentsAndSpaces:(StringInputStream*)dataStream;

- (void)readPropertiesFromStream:(StringInputStream*)dataStream ignoreEncode:(BOOL)ignoreEncode;
- (void)readCommentFromStream:(StringInputStream*)dataStream;
- (void)readPreambleFromStream:(StringInputStream*)dataStream;
- (void)readMacroFromStream:(StringInputStream*)dataStream;
- (void)readBibItem:(NSString*)bibType fromStream:(StringInputStream*)dataStream;

- (NSString*)getBlock:(NSString**)blockData fromStream:(StringInputStream*)dataStream;

- (void)appendError:(NSString*)errorKind withPram1:(NSString*)param1 andParam2:(NSString*)param2;
- (NSArray*)readingErrors;

- (BOOL)removeBibItem:(BibItem*)bibItem removeFile:(BOOL)removeFile;
- (BOOL)moveFileToUserTrash:(NSString*)filePath;

- (NSString*)fullPathOf:(NSString*)file;
- (NSString*)relativePathOf:(NSString*)file;

- (BOOL)saveDatabase;
- (BOOL)saveDatabaseAs:(NSString*)newPathName;
- (NSString*)databasePropertyStringWithLF:(NSString*)lineFeed;
- (NSData*)makeContentsDataForSave;

- (void)remakeFilePath;
- (void)remakeDescription;

- (BOOL)isDirty;

- (NSString*)appendAttachments:(NSArray*)files forItem:(BibItem*)bibItem append:(BOOL)append;
- (NSString*)createTargetFolder:(NSString*)targetFolder withName:(NSString*)baseName;
- (BOOL)checkAttachmentFolder:(NSString*)folder;
- (NSString*)nameForFile:(NSString*)name;
- (BOOL)checkOverwriteFile:(NSString*)file;

@end
