//
//  MacroList.h
//  BibCompanion
//
//  Created by Hitoshi Nakahara on 05/01/22.
//  Copyright 2005 Hit-C Soft House. All rights reserved.
//

#import <Cocoa/Cocoa.h>

#ifndef	BC_MacroDefIndex

typedef enum BC_MacroDefIndex {
	BC_MacroDefIndex_Names = 0,
	BC_MacroDefIndex_Data
} BC_MacroDefIndex;

typedef enum BC_AppendSameMacroAction {
	BC_AppendSameMacroAction_createUniqueName = 0,
	BC_AppendSameMacroAction_useDestination,
	BC_AppendSameMacroAction_useSource,
	BC_AppendSameMacroAction_askUser
} BC_AppendSameMacroAction;

#endif

@class	StringInputStream;

@interface MacroList : NSObject {

	NSMutableArray			*macroNames;			// macro name array to define its order
	NSMutableDictionary		*macroDataForEdit;		// NSString for edit (starts with @ if it has sub macro)
	NSMutableDictionary		*macroDataInTeX;		// expanded macro in TeX format
	NSMutableDictionary		*macroData;				// expanded macro with converted TeX command
	
	BOOL		common;
}

+ (MacroList*)macroList;
+ (MacroList*)macroListWithList:(MacroList*)sourceList;
+ (MacroList*)macroListWithStringList:(NSString**)stringList common:(BOOL)isCommon;
+ (MacroList*)macroListWithMacroDefs:(NSArray*)macroDef common:(BOOL)isCommon;
+ (MacroList*)macroListFromBstFile:(NSString*)bstFile;
- (id)initWithList:(MacroList*)sourceList;
- (id)initWithStringList:(NSString**)stringList common:(BOOL)isCommon;
- (id)initWithMacroDefs:(NSArray*)macroDef common:(BOOL)isCommon;
- (id)initWithBstFile:(NSString*)bstFile;

- (BOOL)common;
- (NSMutableArray*)macroNameList;
- (NSMutableDictionary*)macroDataForEdit;
- (NSArray*)macroDefinition;

- (BOOL)setMacroWithName:(NSString*)macro andData:(NSString*)data;
- (BOOL)appendMacroWithDictionary:(NSDictionary*)macroDic errorStr:(NSString**)errorStr;
- (void)appendMacroWithDefs:(NSArray*)macroDefs options:(BC_AppendSameMacroAction)action;
- (void)replaceMacrosWithDefs:(NSArray*)macroDefs;

- (void)removeAllMacros;
- (BOOL)removeMacro:(NSString*)macro;

- (NSString*)uniqueMacroName:(NSString*)name;
- (int)findIndexOfMacro:(NSString*)name;
- (int)count;

+ (NSString*)dataForEditWithElements:(NSArray*)elements;

- (NSString*)expandMacroData:(NSString*)data ofMacro:(NSString*)macro;
- (NSString*)expandMacroElements:(NSArray*)elements maxIndex:(int)maxIndex;

- (NSString*)expandMacro:(NSString*)macro searchCommon:(BOOL)searchCommon;
- (NSString*)expandMacroInTeX:(NSString*)macro searchCommon:(BOOL)searchCommon;
- (NSString*)macroForEdit:(NSString*)macro;

- (void)sortByName;

+ (NSString*)validateMacroName:(NSString*)inData error:(BOOL*)error;
+ (NSString*)validateMacroData:(NSString*)inData error:(BOOL*)error;

@end
