//
//  RealDatabase.h
//  BibCompanion
//
//  Created by Hitoshi Nakahara on 04/11/20.
//  Copyright 2004 Hit-C Soft House. All rights reserved.
//

#import <Cocoa/Cocoa.h>
#import	"BibDatabase.h"

@class	BibItem, MacroList;

@interface RealDatabaseProperties : DatabaseProperties {
}

- (id)initWithDatabase:(BibDatabase*)db;
- (void)readFromDictionary:(NSDictionary*)propertyDic;
- (void)makeCopyOf:(RealDatabaseProperties*)source;
- (NSMutableDictionary*)makePropertyDictionary;

@end

//////////

@interface RealDatabase : BibDatabase {

	NSMutableDictionary		*bibKeyList;		// dictionary of BibItem for bibkey

	NSMutableArray			*commentList;
	NSMutableArray			*preambleList;
	MacroList				*localMacro;
}

- (NSMutableArray*)preambleList;
- (BOOL)editPreambles;
- (void)preamblesChanged;

- (MacroList*)macroList;
- (BOOL)editMacros;
- (void)macroListChanged;

- (NSStringEncoding)encoding;

- (BibItem*)bibItemForKey:(NSString*)bibkey;

- (BOOL)removeBibItem:(BibItem*)bibItem removeFile:(BOOL)removeFile;
- (void)replaceBibKey:(BibItem*)bibItem;
- (BibItem*)appendNewBibItemWithType:(NSString*)bibtype;
- (BOOL)appendBibItem:(BibItem*)bibItem;
- (BibItem*)addCopyOfBibItem:(BibItem*)bibItem;

- (void)applyMacrosToBibitems;

- (void)cleanForCancelLoading;

- (NSString*)fullPathOf:(NSString*)file;
- (NSString*)relativePathOf:(NSString*)file;

- (NSString*)databaseCommentsStringWithLF:(NSString*)lineFeed;
- (NSString*)databasePreamblesStringWithLF:(NSString*)lineFeed;
- (NSString*)databaseMacroStringWithLF:(NSString*)lineFeed;

- (NSString*)uniqueBibKey:(NSString*)baseKey forItem:(BibItem*)bibItem;

@end
